local StoneGateV5 = ...

-- Apply building asset processor
StoneGateV5:registerAssetProcessor("models/basic/basicSmallGate.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

-- Register Small Gate materials
StoneGateV5:registerAssetId("models/basic/basicSmallGate.fbx/Materials/redRoof", StoneGateSetPrefix .. "_SGATE_RED_ROOF_MATERIAL")
StoneGateV5:registerAssetId("models/basic/basicSmallGate.fbx/Materials/blueRoof", StoneGateSetPrefix .. "_SGATE_BLUE_ROOF_MATERIAL")
StoneGateV5:registerAssetId("models/basic/basicSmallGate.fbx/Materials/brownRoof", StoneGateSetPrefix .. "_SGATE_BROWN_ROOF_MATERIAL")
StoneGateV5:registerAssetId("models/basic/basicSmallGate.fbx/Materials/greyRoof", StoneGateSetPrefix .. "_SGATE_GREY_ROOF_MATERIAL")
StoneGateV5:registerAssetId("models/basic/basicSmallGate.fbx/Materials/grey2Roof", StoneGateSetPrefix .. "_SGATE_GREY2_ROOF_MATERIAL")

-- Register Small Gate prefab nodes
StoneGateV5:registerAssetId("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_0_SGate_RootPart", StoneGateSetPrefix .. "_SGATE_ROOT_PREFAB")
StoneGateV5:registerAssetId("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_0_SGate_RootPart/Pillar1_SGate_Part", StoneGateSetPrefix .. "_SGATE_PILLAR1_PREFAB")
StoneGateV5:registerAssetId("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_0_SGate_RootPart/Pillar2_SGate_Part", StoneGateSetPrefix .. "_SGATE_PILLAR2_PREFAB")
StoneGateV5:registerAssetId("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_1_SGate_Tiling1Part", StoneGateSetPrefix .. "_SGATE_TILING1_PREFAB")
StoneGateV5:registerAssetId("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_1_SGate_ext_Tiling1Part", StoneGateSetPrefix .. "_SGATE_EXT_TILING1_PREFAB")
StoneGateV5:registerAssetId("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_2_SGate_Tiling2Part", StoneGateSetPrefix .. "_SGATE_TILING2_PREFAB")
StoneGateV5:registerAssetId("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_2_SGate_Tiling3Part", StoneGateSetPrefix .. "_SGATE_TILING3_PREFAB")
StoneGateV5:registerAssetId("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_3_SGate_TopPart", StoneGateSetPrefix .. "_SGATE_TOP_PREFAB")
StoneGateV5:registerAssetId("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_3_SGate_TopPPart", StoneGateSetPrefix .. "_SGATE_TOP_P_PREFAB")
--------------------------------------------------------------------------

-- Create default building part registering function 
function registerDefaultBuildingPart2(_nodePrefix)
	StoneGateV5:register({
		DataType = "BUILDING_PART",
		Id = _nodePrefix .. "_PART",
		ConstructorData = {
                    DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
                    CoreObjectPrefab = _nodePrefix .. "_PREFAB"
                },
	        Cost = {
		    UpkeepCost = {
			{ Resource = "GOLD", Quantity = 1 }
		    },
		    RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		    }
	        },
	        EstateSplendor = {
		    { Estate = "KINGDOM", Quantity = 0.1 }
	    }
	})
end

local defaultNodePrefixList2 = {
        StoneGateSetPrefix .. "_SGATE_PILLAR1",
        StoneGateSetPrefix .. "_SGATE_PILLAR2",
	StoneGateSetPrefix .. "_SGATE_TOP",
	StoneGateSetPrefix .. "_SGATE_TOP_P",
	StoneGateSetPrefix .. "_SGATE_TILING1",
	StoneGateSetPrefix .. "_SGATE_EXT_TILING1",
	StoneGateSetPrefix .. "_SGATE_TILING2",
	StoneGateSetPrefix .. "_SGATE_TILING3"
}

-- Register simple building part assets
for i, nodePrefix in ipairs(defaultNodePrefixList2) do
	registerDefaultBuildingPart2(nodePrefix)
end

-- Register the stone gate set small gate root part
StoneGateV5:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_SGATE_ROOT_PART",
	Name = StoneGateSetPrefix .. "_SGATE_ROOT_NAME",
	Description = StoneGateSetPrefix .. "_SGATE_ROOT_DESC",
	Category = "CORE",
        BuildingZone = _SGateBuildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_SGATE_ROOT_PREFAB",
		EndPart = StoneGateSetPrefix .. "_SGATE_TOP_PART",
		FillerList = {
			StoneGateSetPrefix .. "_SGATE_TILING2_PART",
			StoneGateSetPrefix .. "_SGATE_TILING3_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_SGATE_TILING1_PART",
			StoneGateSetPrefix .. "_BGATE_TILING1_PART",
			StoneGateSetPrefix .. "_BGATE_TILING1_PART",
			StoneGateSetPrefix .. "_BGATE_TILING1_PART"
         	},
		MinimumScale = 0,
		IsVertical = true,
                IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 1 }
	}
})

-- Register the stone gate set small gate extension root part
StoneGateV5:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_SGATE_EXT_ROOT_PART",
	Name = StoneGateSetPrefix .. "_SGATE_EXT_ROOT_NAME",
	Description = StoneGateSetPrefix .. "_SGATE_EXT_ROOT_DESC",
	Category = "CORE",
        BuildingZone = _SGateExtensionBuildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_SGATE_ROOT_PREFAB",
		EndPart = StoneGateSetPrefix .. "_SGATE_TOP_PART",
		FillerList = {
			StoneGateSetPrefix .. "_SGATE_TILING2_PART",
			StoneGateSetPrefix .. "_SGATE_TILING3_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_SGATE_EXT_TILING1_PART",
			StoneGateSetPrefix .. "_BGATE_TILING1_PART",
			StoneGateSetPrefix .. "_BGATE_TILING1_PART",
			StoneGateSetPrefix .. "_BGATE_TILING1_PART"
         	},
		MinimumScale = 0,
		IsVertical = true
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 1 }
	}
})

-- Register the stone gate set small gate root wooden floor part
StoneGateV5:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_SGATE_ROOT_P_PART",
	Name = StoneGateSetPrefix .. "_SGATE_ROOT_P_NAME",
	Description = StoneGateSetPrefix .. "_SGATE_ROOT_P_DESC",
	Category = "CORE",
        BuildingZone = _SGateBuildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_SGATE_ROOT_PREFAB",
		EndPart = StoneGateSetPrefix .. "_SGATE_TOP_P_PART",
		FillerList = {
			StoneGateSetPrefix .. "_SGATE_TILING2_PART",
			StoneGateSetPrefix .. "_SGATE_TILING3_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_SGATE_TILING1_PART",
			StoneGateSetPrefix .. "_BGATE_TILING1_PART",
			StoneGateSetPrefix .. "_BGATE_TILING1_PART",
			StoneGateSetPrefix .. "_BGATE_TILING1_PART"
         	},
		MinimumScale = 0,
		IsVertical = true,
                IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 1 }
	}
})

-- Register the stone gate set small gate extension root wooden floor part
StoneGateV5:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_SGATE_EXT_ROOT_P_PART",
	Name = StoneGateSetPrefix .. "_SGATE_EXT_ROOT_P_NAME",
	Description = StoneGateSetPrefix .. "_SGATE_EXT_ROOT_P_DESC",
	Category = "CORE",
        BuildingZone = _SGateExtensionBuildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_SGATE_ROOT_PREFAB",
		EndPart = StoneGateSetPrefix .. "_SGATE_TOP_P_PART",
		FillerList = {
			StoneGateSetPrefix .. "_SGATE_TILING2_PART",
			StoneGateSetPrefix .. "_SGATE_TILING3_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_SGATE_EXT_TILING1_PART",
			StoneGateSetPrefix .. "_BGATE_TILING1_PART",
			StoneGateSetPrefix .. "_BGATE_TILING1_PART",
			StoneGateSetPrefix .. "_BGATE_TILING1_PART"
         	},
		MinimumScale = 0,
		IsVertical = true
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 1 }
	}
})

--------------------------------------------------------------------------

-- Register flag assigner component on the collider's type to 'PLATFORM'

--StoneGateV5:configurePrefabFlagList("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_3_SGate_TopPart", { "PLATFORM" })
 
-- Register grass clearing components

StoneGateV5:registerPrefabComponent("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_0_SGate_RootPart", {
	DataType = "COMP_DIRT_RECTANGLE",
	Size = { 4, 8 }
})

function registerRampartPart(_nameFbx, _nameAsset)
    StoneGateV5:registerAssetId("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_4_SGate_" .. _nameFbx .. "Part", StoneGateSetPrefix .. "_SGATE_" .. _nameAsset .. "_PREFAB")

-- Register the stone gate set decoration rampart part
    StoneGateV5:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_SGATE_" .. _nameAsset .. "_PART",
	Name = StoneGateSetPrefix .. "_SGATE_" .. _nameAsset .. "_NAME",
	Description = StoneGateSetPrefix .. "_SGATE_" .. _nameAsset .. "_DESC",
	Category = "EXTENSION",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_SGATE_" .. _nameAsset .. "_PREFAB",
                IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		}
	}
    })

    StoneGateV5:registerPrefabComponent("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_4_SGate_" .. _nameFbx .. "Part", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = "MINOR"
    })
end

registerRampartPart( "RampartA", "RAMPART_A" )
registerRampartPart( "RampartClosingL", "RAMPART_CLOSING_L" )
registerRampartPart( "RampartClosingR", "RAMPART_CLOSING_R" )

