local drawBridge = ...

DrawBridgePrefix = StoneGateSetPrefix .. "_DRAW_BRIDGE"

drawBridge:registerAssetProcessor("models/drawBridges/drawBridgeA.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
drawBridge:registerAssetProcessor("models/drawBridges/drawBridgeB.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
drawBridge:registerAssetProcessor("models/drawBridges/harrow.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

-- Register bridge tiling (the pillar part that extends to the ground)
drawBridge:registerAssetId("models/drawBridges/drawBridgeA.fbx/Prefab/drawBridge/TilingPart", DrawBridgePrefix .. "_TILING_PREFAB")
drawBridge:register({
	DataType = "BUILDING_PART",
	Id = DrawBridgePrefix .. "_TILING_PART",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = DrawBridgePrefix .. "_TILING_PREFAB"
	},
	Cost = {
		DataType = "BUILDING_PART_COST",
		RessourcesNeeded = {
			{ Resource = "STONE", Quantity = 1 }
		}
	},
})

function registerExtremityPart(_nameFbx, _extremity, _upperAttachType, _lowerAttachType)
    -- Register bridge extremity part
    drawBridge:registerAssetId("models/drawBridges/drawBridgeA.fbx/Prefab/drawBridge/" .. _nameFbx .. "Part", DrawBridgePrefix .. "_" .. _extremity .. "_PREFAB")
    drawBridge:register({
	    DataType = "BUILDING_PART",
	    Id = DrawBridgePrefix .. "_" .. _extremity .. "_PART",
	    ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SLOPE",
		CoreObjectPrefab = DrawBridgePrefix .. "_" .. _extremity .. "_PREFAB",
		SlopePartList = {
			DrawBridgePrefix .. "_SLOPE_" .. _extremity .. "_PART"
		},
		CoreBasementPartList = {
			DrawBridgePrefix .. "_TILING_PART"
		},
		UpperAttachType = _upperAttachType,
		LowerAttachType = _lowerAttachType
            },
	    Cost = {
		DataType = "BUILDING_PART_COST",
		RessourcesNeeded = {
			{ Resource = "STONE", Quantity = 2 }
		}
	    }
    })

    -- Register bridge slope part
    drawBridge:registerAssetId("models/drawBridges/drawBridgeA.fbx/Prefab/drawBridge/" .. _nameFbx .. "SlopePart", DrawBridgePrefix .. "_SLOPE_" .. _extremity .. "_PREFAB")
    drawBridge:register({
	DataType = "BUILDING_PART",
	Id = DrawBridgePrefix .. "_SLOPE_" .. _extremity .. "_PART",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_BASEMENT",
		CoreObjectPrefab = DrawBridgePrefix .. "_SLOPE_" .. _extremity .. "_PREFAB",
		FillerList = {
			DrawBridgePrefix .. "_TILING_PART"
		}
	},
	Cost = {
		DataType = "BUILDING_PART_COST",
		RessourcesNeeded = {
			{ Resource = "STONE", Quantity = 1 }
		}
        }
    })

    -- Configure BRIDGE colliders
    drawBridge:configurePrefabFlagList("models/drawBridges/drawBridgeA.fbx/Prefab/drawBridge/" .. _nameFbx .. "Part", { "BRIDGE" })
    drawBridge:configurePrefabFlagList("models/drawBridges/drawBridgeA.fbx/Prefab/drawBridge/" .. _nameFbx .. "SlopePart", { "BRIDGE" })
end

registerExtremityPart("Start", "START", ATTACH_NODE_TYPE.RIGHT, ATTACH_NODE_TYPE.LEFT)
registerExtremityPart("End", "END", ATTACH_NODE_TYPE.LEFT, ATTACH_NODE_TYPE.RIGHT)

function registerCenterParts(_nameFbx, _option)
    -- Register bridge center top part
    drawBridge:registerAssetId("models/drawBridges/drawBridge" .. _option .. ".fbx/Prefab/drawBridge/sgv5db" .. _nameFbx .. "_CenterPart", DrawBridgePrefix .. "_" .. _option .. "_CENTER_PREFAB")
    drawBridge:register({
	DataType = "BUILDING_PART",
	Id = DrawBridgePrefix .. "_" .. _option .. "_CENTER_TOP_PART",
        BuildingZone = _DrawBridgeBuildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = DrawBridgePrefix .. "_" .. _option .. "_CENTER_PREFAB"
	},
	Cost = {
		DataType = "BUILDING_PART_COST",
		RessourcesNeeded = {
			{ Resource = "TOOL", Quantity = 2 },
			{ Resource = "PLANK", Quantity = 10 }
                }
        }
    })

    -- Register the center root part (the scalable part that places multiple center top parts)
    drawBridge:registerAssetId("models/drawBridges/drawBridgeA.fbx/Prefab/drawBridge/RootPart", DrawBridgePrefix .. "_CENTER_ROOT_PREFAB")
    drawBridge:register({
	DataType = "BUILDING_PART",
	Id = DrawBridgePrefix .. "_" .. _option .. "_CENTER_CORE_PART",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER",
		CoreObjectPrefab = DrawBridgePrefix .. "_CENTER_ROOT_PREFAB",
		EndPart = DrawBridgePrefix .. "_" .. _option .. "_CENTER_TOP_PART",
		FillerList = {
			DrawBridgePrefix .. "_TILING_PART"
		}
	},
	Cost = {
		DataType = "BUILDING_PART_COST",
		RessourcesNeeded = {
			{ Resource = "PLANK", Quantity = 1 }
                }
        }
    })

    -- Register the core part (the part with the bridge mover)
    drawBridge:register({
	DataType = "BUILDING_PART",
	Id = DrawBridgePrefix .. "_" .. _option .. "_CORE_PART",
	Name = DrawBridgePrefix .. "_" .. _option .. "_CORE_NAME",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_BRIDGE",
		StartPart = DrawBridgePrefix .. "_START_PART",
		EndPart = DrawBridgePrefix .. "_END_PART",
		FillerList = {
			DrawBridgePrefix .. "_" .. _option .. "_CENTER_CORE_PART"
		},
		MinimumScale = 1,
                MinimumHeightAboveGround = 0
	},
	AssetBuildingFunction = DrawBridgePrefix .. "_FUNCTION"
    })

    -- Configure BRIDGE collider
    drawBridge:configurePrefabFlagList("models/drawBridges/drawBridge" .. _option .. ".fbx/Prefab/drawBridge/sgv5db" .. _nameFbx .. "_CenterPart", { "BRIDGE" })
end

registerCenterParts("a", "A") -- simple draw bridge
registerCenterParts("b", "B") -- draw bridge with arrows

drawBridge:register({
		DataType = "BUILDING_FUNCTION_BRIDGE",
                Id = DrawBridgePrefix .. "_FUNCTION"
})

-- Register the additional harrow part
drawBridge:registerAssetId("models/drawBridges/harrow.fbx/Prefab/drawBridge/HarrowPart", DrawBridgePrefix .. "_HARROW_PREFAB")
drawBridge:register({
	DataType = "BUILDING_PART",
	Id = DrawBridgePrefix .. "_HARROW_PART",
	Name = DrawBridgePrefix .. "_HARROW_NAME",
	Description = DrawBridgePrefix .. "_HARROW_DESC",
	Category = "EXTENSION",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = DrawBridgePrefix .. "_HARROW_PREFAB",
                IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "TOOL", Quantity = 1 }
		}
	}
})
